"use strict";
/**************************************************************************************************

	This software package is released under the Modified BSD License.

	Eclipsed Moon

		Copyright © 2017, Eurythrace Perseides. All rights reserved.

		An add-on/extension exclusively for the Pale Moon browser for smart, unobtrusive user
		agent spoofing.

										――――――――――――――――――――

										Modified BSD License

	Redistribution and use in source and binary forms, with or without modification, are permitted
	provided that the following conditions are met:

	1.	Redistributions of source code must retain the above copyright notices, this list of
		conditions and the following disclaimer.

	2.	Redistributions in binary form must reproduce the above copyright notice, this list of
		conditions and the following disclaimer in the documentation and/or other materials
		provided with the distribution.

	3.	Neither the name of the copyright holder nor the names of its contributors may be used to
		endorse or promote products derived from this software without specific prior written
		permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS	“AS IS” AND ANY EXPRESS OR
	IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
	AND FITNESS FOR	A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
	CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
	CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
	SERVICES; LOSS OF USE, DATA, OR	PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
	OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
	POSSIBILITY OF SUCH DAMAGE.

**************************************************************************************************/

const { classes: Cc, interfaces: Ci, results: Cr, utils: Cu } = Components;

const empDebug = false;

//-------------------------------------------------------------------------------------------------
// Define the constructor for the user agent spoofer preferences object. 

var UmCastPrefs = ( function()
{
	const nsXHTML = "http://www.w3.org/1999/xhtml";
	const PREF_TYPE_INVALID = 0, PREF_TYPE_STRING = 32, PREF_TYPE_INT = 64, PREF_TYPE_BOOL = 128;

	var PrefService		= Cc["@mozilla.org/preferences-service;1"].getService( Ci.nsIPrefService ),
		DefaultPrefs	= PrefService.getDefaultBranch( "" ),
		UserSetPrefs	= PrefService.getBranch( "" );

	var ucp = function(){};

//-------------------------------------------------------------------------------------------------
//	Intialize the preference manipulator object when the prefpanel loads.
	ucp.prototype.startup = function()
	{
if( empDebug ) { console.log( ">>>> EclipsePrefs: Entering the startup routine." ); }
	//	Make a preference list.
		let list = document.getElementsByTagName( "preference" );
		this.preferences = {};
		for( let i = 0; i < list.length; i++ )
		{
			this.preferences[list.item( i ).id] = { "name": list.item( i ).name, "type": list.item( i ).type };
		}
	};

//-------------------------------------------------------------------------------------------------
//	Reset all preferences on a tabpanel to their default values.
	ucp.prototype.resetToDefaults = function( element )
	{
		let tabpanel = element, prefName, prefValue, suggested;
		while( tabpanel.tagName !== "tabpanel" ) { tabpanel = tabpanel.parentNode; }
if( empDebug ) { console.log( ">>>> EclipsePrefs: resetToDefaults: tabpanel ID = “" + tabpanel.id + "”." ); }

	//	Make a checkbox preference list.
		let list = tabpanel.getElementsByTagName( "checkbox" );
		for( let i = 0, checkbox; i < list.length; i++ )
		{
			checkbox = list.item( i );
			prefName = this.preferences[checkbox.getAttribute( "preference" )].name;

			if( DefaultPrefs.getPrefType( prefName ) !== PREF_TYPE_INVALID )
			{
				prefValue = DefaultPrefs.getBoolPref( prefName );
			}
			else
			{
				suggested = checkbox.parentNode.getElementsByTagNameNS( nsXHTML, "span" ).item( 0 ).firstChild.nodeValue;
				prefValue = ( ( suggested === "☑" ) ? true : false );
			}
			if( checkbox.checked !== prefValue ) { checkbox.click(); }
		}

	//	Make a textbox preference list.
		list = tabpanel.getElementsByTagName( "textbox" );
		for( let i = 0, textbox; i < list.length; i++ )
		{
			textbox = list.item( i );
			prefName = this.preferences[textbox.getAttribute( "preference" )].name;

			if( DefaultPrefs.getPrefType( prefName ) !== PREF_TYPE_INVALID )
			{
				if( textbox.type === "number" )
				{
					prefValue = DefaultPrefs.getIntPref( prefName );
				}
				else
				{
					prefValue = DefaultPrefs.getCharPref( prefName );
				}
			}
			else
			{
				suggested = textbox.parentNode.getElementsByTagNameNS( nsXHTML, "span" ).item( 0 ).firstChild.nodeValue;
				if( textbox.type === "number" )
				{
					prefValue = 0;
					if( suggested !== "–" ) { prefValue = parseInt( suggested, 10 ); }
				}
				else
				{
					prefValue = ( ( suggested === "\"\"" || suggested === "–" ) ? "" : suggested );
				}
			}

			if( textbox.type === "number" )
			{
				textbox.valueNumber = prefValue;
			}
			else
			{
				textbox.value = prefValue;
			}
			let deed = document.createEvent( "UIEvent" );
			deed.initEvent( "change", true, true );
			textbox.dispatchEvent( deed );
		}

	//	Make a radiogroup preference list.
		list = tabpanel.getElementsByTagName( "radiogroup" );
		for( let i = 0, radiogroup, radios; i < list.length; i++ )
		{
			radiogroup = list.item( i );
			prefName = this.preferences[radiogroup.getAttribute( "preference" )].name;

			if( DefaultPrefs.getPrefType( prefName ) !== PREF_TYPE_INVALID )
			{
				prefValue = DefaultPrefs.getCharPref( prefName );
			}
			else
			{
				suggested = radiogroup.parentNode.getElementsByTagNameNS( nsXHTML, "span" ).item( 0 ).firstChild.nodeValue;
				prefValue = ( ( suggested === "\"\"" || suggested === "–" ) ? "" : suggested );
			}

			if( prefValue === "" ) { continue; }

		//	Special case for installation!
			if( prefValue === "fresh" ) { prefValue = "smart"; }

			radios = radiogroup.getElementsByTagName( "radio" );
			for( let j = 0; j < radios.length; j++ )
			{
				if( radios.item( j ).value === prefValue )
				{
					radios.item( j ).click();
					break;
				}
			}
		}
	};

//-------------------------------------------------------------------------------------------------
//	Set all preferences on a tabpanel to their suggested values.
	ucp.prototype.setToSuggested = function( element )
	{
		let tabpanel = element, suggested;
		while( tabpanel.tagName !== "tabpanel" ) { tabpanel = tabpanel.parentNode; }
if( empDebug ) { console.log( ">>>> EclipsePrefs: setToSuggested: tabpanel ID = “" + tabpanel.id + "”." ); }

	//	Make a checkbox preference list.
		let list = tabpanel.getElementsByTagName( "checkbox" );
		for( let i = 0, checkbox; i < list.length; i++ )
		{
			checkbox = list.item( i );
			suggested = checkbox.parentNode.getElementsByTagNameNS( nsXHTML, "span" ).item( 0 ).firstChild.nodeValue;

			switch( suggested )
			{
			case "–":
				continue;
			case "☑":
				suggested = true;
				break;
			default:
				suggested = false;
			}
			if( checkbox.checked !== suggested ) { checkbox.click(); }
		}

	//	Make a textbox preference list.
		list = tabpanel.getElementsByTagName( "textbox" );
		for( let i = 0, textbox; i < list.length; i++ )
		{
			textbox = list.item( i );
			suggested = textbox.parentNode.getElementsByTagNameNS( nsXHTML, "span" ).item( 0 ).firstChild.nodeValue;

			switch( suggested )
			{
			case "–":
				continue;
			case "\"\"":
				textbox.value = "";
				break;
			default:
				if( textbox.type === "number" )
				{
					textbox.valueNumber = parseInt( suggested, 10 );
				}
				else
				{
					textbox.value = suggested;
				}
			}
			let deed = document.createEvent( "UIEvent" );
			deed.initEvent( "change", true, true );
			textbox.dispatchEvent( deed );
		}

	//	Make a radiogroup preference list.
		list = tabpanel.getElementsByTagName( "radiogroup" );
		for( let i = 0, radiogroup, radios; i < list.length; i++ )
		{
			radiogroup = list.item( i );
			suggested = radiogroup.parentNode.getElementsByTagNameNS( nsXHTML, "span" ).item( 0 ).firstChild.nodeValue;

			if( suggested === "\"\"" || suggested === "–" ) { continue; }

			radios = radiogroup.getElementsByTagName( "radio" )
			for( let j = 0; j < radios.length; j++ )
			{
				if( radios.item( j ).value === suggested )
				{
					radios.item( j ).click();
					break;
				}
			}
		}
	};

//-------------------------------------------------------------------------------------------------
//	All done with construction.
	return ucp;
} )();

var EclipsePrefs = new UmCastPrefs();
